/*
 * Process with rank 0 scatters a message to all processes via MPI_COMM_WORLD 
 * and then gathers back the message. 
 *
 */

/* The definition of MPI */
#include "mpi.h"


void main( int argc, char **argv )
{
	int source_value = 42, ret_val = -1; /* sending and return value to/from the process */
	int root = 0, tag = 0;
	int my_rank, num_processes;

	double start, finish, time;
	MPI_Status status;

	/* start up MPI */
	MPI_Init( &argc, &argv );

	/* get process rank */
	MPI_Comm_rank( MPI_COMM_WORLD, &my_rank );

	/* get number of processes */
	MPI_Comm_size( MPI_COMM_WORLD, &num_processes );

	/* set up the starting time */
	start = MPI_Wtime();
  
	/* Process with rank 0 scatters a message to all processes 
	   and then gathers back the message. */
	MPI_Scatter( &source_value, 1, MPI_INT, &ret_val, 1, MPI_INT, root, MPI_COMM_WORLD );
	MPI_Gather( &source_value, 1, MPI_INT, &ret_val, 1, MPI_INT, root, MPI_COMM_WORLD );

	/* stop the time */
	finish = MPI_Wtime();

	time = finish - start;

	/* print out the result */
	if ( my_rank == root )
	{
		printf( "Measurement Time with Scatter and Gather = %f\n", time );
	}

	/* close MPI */
	MPI_Finalize();
}
